
globals [ zoom-factor ] ;; used to "zoom" by scaling the displayed positions of particles
breed [ stayers stayer ] ;; used as points to form the mesh that displays the gravity wells
breed [ particles particle ] ;; particles that interact and exert gravitational forces upon each other
turtles-own [
  x-pos y-pos z-pos ;; used as the basis for physics calculations - xcor, ycor, zcor only used for display
]
stayers-own [
  gravity ;; the magnitude of the total gravitational field at a given point
]
particles-own [
  x-velocity y-velocity ;; components of 2d velocity, used for physics calculations
  mass ;; also used for physics; the greater the mass, the greater the gravitational pull
]

to setup
  clear-all

  ;; create stayers
  ask patches with [pzcor = 0] [
    sprout-stayers 1
    [
      set gravity 0
      create-links-with turtles-on neighbors
      set x-pos xcor
      set y-pos ycor
      hide-turtle
    ]
  ]

  set-default-shape turtles "circle"
  set zoom-factor 1
  create-particles number-of-objects [
    setxyz (random-float starting-box - random-float starting-box) (random-float starting-box - random-float starting-box) 0
    set x-velocity ((random-float max-starting-velocity) - (random-float max-starting-velocity))
    set y-velocity ((random-float max-starting-velocity) - (random-float max-starting-velocity))
    ;set z-velocity ((random-float max-starting-velocity) - (random-float max-starting-velocity))
    set mass random-float max-starting-mass
    set size mass ^ (1 / 3)
    set x-pos xcor
    set y-pos ycor
    set z-pos zcor
    ]
  compute-gravity-wells
  set-particle-display-positions
  reset-ticks
end

to go
  compute-acceleration-changes
  check-boundaries
  move-particles
  check-slide
  compute-gravity-wells
  set-particle-display-positions
  tick
end


to tick-once
  compute-acceleration-changes
  check-boundaries
  move-particles
  check-slide
  compute-gravity-wells
  set-particle-display-positions
  tick
end

;; Update positions of stayers to correspond with gravitational field
to compute-gravity-wells

  ask stayers [ set gravity 0
    set x-pos xcor * zoom-factor
    set y-pos ycor * zoom-factor
  ]
  ask particles [
    ask stayers with [distancexy [xcor] of myself [ycor] of myself < SoI] [
      let xdist [x-pos] of self - [x-pos] of myself
      let ydist [y-pos] of self - [y-pos] of myself
      let scale-const max list ((xdist ^ 2 + ydist ^ 2) ^ .5) ([size] of myself / 2)

      set gravity gravity + [mass] of myself * G / scale-const
    ]
  ]
  ask stayers [
    set z-pos gravity * -1
    set zcor z-pos / zoom-factor
    ; set color scale-color red zcor 0 -50
  ]
end

to set-particle-display-positions
  ask particles [
    set z-pos [z-pos] of min-one-of stayers [distancexy [xcor] of myself [ycor] of myself]
    set zcor z-pos / zoom-factor

    if not hidden? [
      set xcor x-pos / zoom-factor
      set ycor y-pos / zoom-factor
      set size mass ^ (1 / 3) / zoom-factor
    ]
  ]
end

;; Check center of mass of the overall system, adjust view accordingly
to check-slide
  let CoM center-of-mass particles
  slide-view (item 0 CoM) (item 1 CoM) (0)
end

;; Calculate the center of mass of the entire system
to-report center-of-mass [turtleset]
  ; R = 1/M sigma (mi * ri)
  let M 0
  let totalx 0
  let totaly 0
  ;let totalz 0
  ask turtleset [
    let mi [mass] of self
    set M M + mi
    set totalx totalx + [x-pos] of self * mi
    set totaly totaly + [y-pos] of self * mi
    ;set totalz totalz + [z-pos] of self * mi
  ]

  report (list (totalx / M) (totaly / M))

end

to check-boundaries
  let flag false
  ask particles [
    if abs (x-pos + x-velocity) > max-pxcor * zoom-factor * .8 or
    abs (y-pos + y-velocity) > max-pycor * zoom-factor * .8; or
    ;abs (z-pos + z-velocity) > max-pzcor * zoom-factor * .8
    [
      set flag true
    ]
  ]

  ifelse flag [
    ;print "zooming out"
    set-zoom 1.1
  ]
  [
    ;print "zooming in"
    set-zoom .9
  ]

  ask particles [
    ifelse abs (x-pos + x-velocity) > max-pxcor or abs (y-pos + y-velocity) > max-pycor or abs (z-pos) > max-pzcor
    [ set hidden? true ]
    [ set hidden? false ]
  ]


end

;; Have particles compute their acceleration based on gravitational interactions with all other particles
to compute-acceleration-changes
  ask particles [
    let turtle1 self
    ask other particles [
      gravitationally-affect turtle1 self
    ]
  ]
end

;; Move particles according to their velocity
to move-particles
  ask particles [
    set x-pos x-pos + x-velocity
    set y-pos y-pos + y-velocity
    ;set z-pos z-pos + z-velocity
  ]
end

;; Compute gravitational pull between a pair of particles
to gravitationally-affect [ affected affecting ]
  ; g = m1 * m2 / r^2
  ; r = sqrt(xdist^2 + ydist^2) -> r^2 = xdist^2 + ydist^2
  let xdist [x-pos] of affecting - [x-pos] of affected
  ;print xdist
  let ydist [y-pos] of affecting - [y-pos] of affected
  ;print ydist
  ;let zdist [z-pos] of affecting - [z-pos] of affected
  let total-acc 0
  let xyangle 90
  ;let zangle 90
  let xydist (xdist ^ 2 + ydist ^ 2) ^ .5
  if xdist + ydist > 0 [
    set total-acc [mass] of affecting / (xdist ^ 2 + ydist ^ 2)
    let angle-heading atan xdist ydist
    set xyangle (90 - angle-heading) mod 360
    ;set zangle (90 - z-angle-heading) mod 360
  ]

  if total-acc > 50 [ set total-acc 50 ]

  set total-acc total-acc / 20
  ;let xyacc total-acc * (cos zangle)

  ask affected [
    set x-velocity (x-velocity + total-acc * (cos xyangle))
    set y-velocity (y-velocity + total-acc * (sin xyangle))
    ;set z-velocity (z-velocity + total-acc * (sin zangle))
  ]

end

;; Move view laterally
to slide-view [deltax deltay deltaz]
  ask particles [
    set x-pos x-pos - deltax
    set y-pos y-pos - deltay
    set z-pos z-pos - deltaz
  ]
end

;; Change zoom-factor to zoom in or out
to set-zoom [new-zoom]
  set zoom-factor zoom-factor * new-zoom
end
@#$#@#$#@
GRAPHICS-WINDOW
0
0
1323
1344
50
50
13.0
1
10
1
1
1
0
1
1
1
-50
50
-50
50
-50
50
1
0
1
ticks
30.0

BUTTON
40
180
115
213
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
125
180
195
213
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
205
180
286
213
NIL
tick-once
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
40
20
285
53
number-of-objects
number-of-objects
0
20
5
1
1
NIL
HORIZONTAL

SLIDER
40
60
285
93
starting-box
starting-box
0
100
29
1
1
NIL
HORIZONTAL

SLIDER
40
100
285
133
max-starting-velocity
max-starting-velocity
0
10
0
.01
1
NIL
HORIZONTAL

SLIDER
40
140
285
173
max-starting-mass
max-starting-mass
0
100
76
1
1
NIL
HORIZONTAL

MONITOR
385
110
465
155
NIL
zoom-factor
3
1
11

SLIDER
315
20
560
53
G
G
0
1
0.5
.01
1
NIL
HORIZONTAL

SLIDER
315
60
560
93
SoI
SoI
0
100
100
1
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

This model is part of the Gravitational Model Suite. This model specifically displays "gravity wells" a visual representation of gravitational fields. While the other models in the suite simulate gravity in 3d space, this model simulates in 2d space, and uses the z axis to display the magnitude of the gravitational field at every point in space.

## HOW IT WORKS

Every particle has mass, position, and velocity. At every time step, every particle pulls on every other particle with a gravitational force proportional to its mass and inversely proportional to the distance between the two particles. Each particle sums up these forces to calculate its total acceleration at every tick, and updates its velocity accordingly.

Uniquely to this model, the magnitude of the gravitational field is also calculated at every point in space, and displayed as the distance the mesh plane is distorted along the z-axis. This provides an intuitive view of the gravity of an object - the deeper of a distortion it makes, the stronger its pull, and being captured by its gravity is analogous to falling down its gravity well.

## HOW TO USE IT

The NUMBER-OF-OBJECTS slider determines how many particles are spawned. The STARTING-BOX slider defines the coordinates they can be spawned inside - a smaller value spawns them closer to the center. MAX-STARTING-VELOCITY allows them to have a randomly determined starting velocity, and MAX-STARTING-MASS defines a maximum mass for the particles to be randomly assigned. SETUP sets up the model, and GO starts or pauses it. G and SoI are for display purposes only - G simply scales the amount that the mesh field is distorted, so higher values of G result in deeper gravity wells. SoI (or "Sphere of Influence") is the distance that a particle will affect visual updates for the gravity wells, so turning down SoI makes the model run faster, but loses visual accuracy farther away from the particles. Note that SoI DOES NOT affect physics calculations at all, only the visual gravity well display. ZOOM-FACTOR simply displays the amount that the model is zoomed in or out.

## THINGS TO NOTICE

Notice that along an orbit, kinetic energy and potential energy are in opposition. When a particle reaches the highest point in its orbit (i.e., has the most potential energy), it is moving the slowest. When the particle reaches its lowest point, it is moving fastest. Conservation of energy states that a particle's total energy (kinetic + potential) must be constant unless it gains or loses energy by interaction with something else. Therefore, as it is moving through its orbit, it converts kinetic energy to potential energy and vice versa.

This also means that if (for example) a particle gains kinetic energy at the lowest point of its orbit, its highest point will correspondingly have more potential energy, which requires it to be farther away. In simpler terms, speeding up widens the opposite end of the orbit, and slowing down brings the opposite end closer. These are all emergent properties that result naturally from Newtonian laws of gravitation.

## THINGS TO TRY

Try messing with G and SoI to make the gravity wells more apparent and visually interesting. Also try letting particles come near each other and seeing how their gravitational fields interact.

## EXTENDING THE MODEL

One of the biggest problems with this model is the simple fact that updating the mesh to display gravity wells is very computationally expensive and takes a long time. There are a bunch of ways that this could be made more efficient - the "sphere of influence" simplification is a pretty naive and simple one. It would also be possible to group together masses that are far away and treat them as one - this would simplify calculations a bit without completely disregarding things beyond an arbitrary distance. Or instead of doing calculations for every single point on the mesh, maybe a subset could be picked and the other point positions could just be extrapolated from there. However, while speeding this up would be great, I decided that accuracy is the #1 priority. This is a model, not a video game.

## NETLOGO FEATURES

One workaround that was used was "zooming" in and out to compensate for the fact that Netlogo doesn't support resizing the world. So when a particle tries to go outside the bounds of the world, rather than making the world bigger, I just scaled down all the positions. This is why every turtle has a separate x-pos, y-pos and z-pos defined in addition to the default xcor, ycor and zcor; the new -pos variables are not scaled down and are used in all physics calculations, while the default cor variables are modified by the global zoom-factor variable and are used for displaying turtles. As a result, the world can effectively become bigger and smaller as needed.

## RELATED MODELS

This model is part of a suite of models, along with the Basic Gravitation model and the Orbital Playground model. The code for the gravity well mesh was also heavily inspired by the code from the Hill Climbing Example 3D model in the Models Library.

## CREDITS AND REFERENCES

http://modelingcommons.org/browse/one_model/4670

Wilensky, U. (1999). NetLogo [computer software]. Evanston, IL: Center for Connected Learning and Computer-Based Modeling, Northwestern University.  http://ccl.northwestern.edu/netlogo .
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3D 6.0-M5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
